/* -------------------------------------------------------------
                         sysf4x1_init.c

   initialisiert MCU STM32F401 / F411 / F407 fuer einen
   "allgemeinen" Gebrauch.

   Das System kann mittels sys_init auf 48, 84, 96, 120  oder
   168MHz Takt eingestellt werden. Die Funktion
                    < sys_tick_handler >
   wird jede ms per Interrupt  ausgefuehrt und in dieser wird
   eine Variable tick_ms  hochgezaehlt.

   Der Takt fuer GPIO-Pins der Ports A,B und C wird einge-
   schaltet und im hier zugehoerigen Header sind mittels
   defines die GPIO - Pins als Input / Output konfigurierbar.

    Hardware  : STM32F401RET / STM32F411
    IDE       : keine (Editor / make)
    Library   : libopencm3
    Toolchain : arm-none-eabi

    30.03.2016   R. seelig

   ------------------------------------------------------------- */

#include "sysf4x1_init.h"

volatile uint32_t tick_ms = 0;

/* -------------------------------------------------------------
                          sys_tick_handler

     Systeminterrupt, wird im Intervall jede Millisekunde auf-
     gerufen und zaehlt die globale Variable tick_ms hoch.
   ------------------------------------------------------------- */
void sys_tick_handler(void)
{
  tick_ms++;
}

/* -------------------------------------------------------------
                                delay

     Zeitverzoegerung, die die Verzoegerungszeit aus dem
     Systemticker bezieht.

     Uebergabe:

          c   : Anzahl zu wartender Millisekunden
   ------------------------------------------------------------- */
void delay(uint32_t c)
{
  volatile uint32_t end_time = tick_ms + c;

  while(tick_ms < end_time)
  {
    __asm volatile("wfi");
  }
}


/* -------------------------------------------------------------
                           systick_setup

     Stellt die Werte fuer einen Systemtimerintervall ein, der
     jede Millisekunde aufgerufen wird.
   ------------------------------------------------------------- */
void systick_setup(uint32_t clock_speed)
{
  // Systemtakt / 1000 erzeugt 1mS Interruptintervall
  systick_set_reload(clock_speed);
  systick_set_clocksource(STK_CSR_CLKSOURCE_AHB);
  systick_counter_enable();
  systick_interrupt_enable();
}


/* -------------------------------------------------------------
                              sys_init

     initialisiert die Taktgeschwindigkeit, mit der der
     Controller betrieben wird.

     Uebergabe:
        osc_speed  :   bestimmt die Grundgeschwindigkeit,
                       zulaessige Werte sind 0 .. 4 oder ueber
                       Enumeratorbezeichnungen:

                         MHZ48, MHZ84, MHZ96, MHZ120, MHZ168

        clock_source :  bestimmt, ob der interne RC-Oszillator
                        oder ein externer 8MHz Quarz verwendet
                        werden soll.

                        Zulaessige Werte sind 0 (intern) oder
                        1 (extern) sowie Enumeratorbezeichnung:

                          intern_clock, extern_clock
   ------------------------------------------------------------- */
void sys_init(uint8_t osc_speed, uint8_t clock_source)
{
  if (clock_source == intern_clock)
  {
    switch (osc_speed)
    {
      case MHZ48 :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_48MHZ]);
        systick_setup(48000);
        break;
      }
      case MHZ84 :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_84MHZ]);
        systick_setup(84000);
        break;
      }
      case MHZ96 :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_96MHZ]);
        systick_setup(96000);
        break;
      }
      case MHZ120 :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_120MHZ]);
        systick_setup(120000);
        break;
      }
      case MHZ168 :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_168MHZ]);
        systick_setup(168000);
        break;
      }
      default :
      {
        rcc_clock_setup_hsi_3v3(&rcc_hsi_3v3[RCC_CLOCK_3V3_48MHZ]);
        systick_setup(48000);
        break;
      }
    }
  }
  else
  {
    switch (osc_speed)
    {
      case MHZ48 :
      {
        rcc_clock_setup_hse_3v3(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_48MHZ]);
        systick_setup(48000);
        break;
      }
      case MHZ84 :
      {
        rcc_clock_setup_hse_3v3(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_84MHZ]);
        systick_setup(84000);
        break;
      }
      case MHZ96 :
      {
        rcc_clock_setup_hse_3v3(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_96MHZ]);
        systick_setup(96000);
        break;
      }
      case MHZ120 :
      {
        rcc_clock_setup_hse_3v3(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_120MHZ]);
        systick_setup(120000);
        break;
      }
      case MHZ168 :
      {
        rcc_clock_setup_hse_3v3(&rcc_hse_8mhz_3v3[RCC_CLOCK_3V3_168MHZ]);
        systick_setup(168000);
        break;
      }
      default :
      {
        systick_setup(48000);
        break;
      }
    }
  }

  // Peripherietakt einschalten
  rcc_periph_clock_enable(RCC_GPIOA);
  rcc_periph_clock_enable(RCC_GPIOB);
  rcc_periph_clock_enable(RCC_GPIOC);
}
